@@ -1,13 +1,15 @@
-from flask import Blueprint, Response, request, stream_with_context
+from urllib.parse import urlparse
 
 import cirrocumulus.data_processing as data_processing
+from cirrocumulus.envir import CIRRO_SERVE
+from flask import Blueprint, Response, request, stream_with_context
+
 from .auth_api import AuthAPI
 from .database_api import DatabaseAPI
 from .dataset_api import DatasetAPI
 from .invalid_usage import InvalidUsage
 from .util import *
 
-
 blueprint = Blueprint('blueprint', __name__)
 
 dataset_api = DatasetAPI()
@@ -86,6 +88,44 @@ def handle_category_name():
         return '', 200
 
 
+@blueprint.route('/feature_set', methods=['POST', 'DELETE'])
+def handle_feature_set():
+    """CRUD for a feature set.
+    """
+
+    email = auth_api.auth()['email']
+    # if request.method == 'GET':
+    #     filter_id = request.args.get('id', '')
+    #     dataset_id = request.args.get('ds_id', '')
+    #     if filter_id == '' or dataset_id == '':
+    #         return 'Please provide an id', 400
+    #     return to_json(database_api.get_dataset_filter(email, dataset_id=dataset_id, filter_id=filter_id))
+    content = request.get_json(force=True, cache=False)
+    set_id = content.get('id')
+    dataset_id = content.get('ds_id')
+    # POST=new, PUT=update , DELETE=delete, GET=get
+    if request.method == 'PUT' or request.method == 'POST':
+        if request.method == 'PUT' and set_id is None:
+            return 'Please supply an id', 400
+        if request.method == 'POST' and dataset_id is None:
+            return 'Please supply a ds_id', 400
+
+        name = content.get('name')
+        category = content.get('category')
+        features = content.get('features')
+        set_id = database_api.upsert_feature_set(
+            email=email,
+            dataset_id=dataset_id,
+            set_id=set_id if request.method == 'PUT' else None,
+            name=name,
+            category=category,
+            features=features)
+        return to_json({'id': set_id})
+    elif request.method == 'DELETE':
+        database_api.delete_feature_set(email, dataset_id=dataset_id, set_id=set_id)
+        return to_json('', 204)
+
+
 @blueprint.route('/filter', methods=['GET', 'POST', 'PUT', 'DELETE'])
 def handle_dataset_filter():
     """CRUD for a dataset filter.
@@ -131,6 +171,7 @@ def handle_schema():
     dataset_id = request.args.get('id', '')
     dataset = database_api.get_dataset(email, dataset_id)
     schema = dataset_api.schema(dataset)
+    schema['markers'] = database_api.get_feature_sets(email=email, dataset_id=dataset_id)
     return to_json(schema)
 
 
@@ -141,9 +182,26 @@ def handle_file():
     dataset = database_api.get_dataset(email, dataset_id)
     file = request.args.get('file')
     url = dataset['url']
+
     import os
     import mimetypes
-    file_path = os.path.join(os.path.dirname(url), os.path.abspath(file))
+    # when serving dataset, image must be relative to dataset directory
+    if os.environ.get(CIRRO_SERVE) == 'true':
+        _, ext = os.path.splitext(url)
+        if ext != '':
+            url = os.path.dirname(url)
+        if file[0] == '/' or file.find('..') != -1:
+            raise ValueError('Incorrect path')
+        file_path = os.path.join(url, file)
+    else:
+        file_path = file
+        pr = urlparse(file_path)
+        if (pr.scheme == '' or pr.scheme == 'file') and not os.path.exists(file_path):
+            _, ext = os.path.splitext(url)
+            if ext != '':
+                url = os.path.dirname(url)
+            file_path = os.path.join(url, file)
+
     mimetype = mimetypes.guess_type(file_path)
     # with dataset_api.fs_adapter.get_fs(file_path).open(file_path) as f:
     #     bytes = f.read()
@@ -226,16 +284,17 @@ def handle_dataset():
     if request.method == 'PUT' or request.method == 'POST':
         content = request.get_json(force=True, cache=False)
         dataset_id = content.get('id', '')
-        if request.method == 'POST' and dataset_id == '':
+        if request.method == 'PUT' and dataset_id == '':
             return 'Please supply an id', 400
         readers = set(content.get('readers', []))
         dataset_name = content.get('name', '')
+        description = content.get('description', '')
         url = content.get('url', '')  # e.g. gs://foo/a/b/
         if dataset_name == '' or url == '':
             return 'Must supply dataset name and URL', 400
         dataset_id = database_api.upsert_dataset(email=email,
             dataset_id=dataset_id if request.method == 'PUT' else None,
-            dataset_name=dataset_name, url=url, readers=readers)
+            dataset_name=dataset_name, url=url, readers=readers, description=description)
         return to_json({'id': dataset_id})
     elif request.method == 'DELETE':
         content = request.get_json(force=True, cache=False)
