@@ -64,8 +64,9 @@ class DatabaseAPI:
     def get_dataset(self, email, dataset_id, ensure_owner=False):
         return self.provider.get_dataset(email, dataset_id, ensure_owner)
 
-    def upsert_dataset(self, email, dataset_id, dataset_name, url, readers):
-        return self.provider.upsert_dataset(email, dataset_id, dataset_name, url, readers)
+    def upsert_dataset(self, email, dataset_id, dataset_name, url, readers, description):
+        return self.provider.upsert_dataset(email=email, dataset_id=dataset_id, dataset_name=dataset_name, url=url,
+            readers=readers, description=description)
 
     def dataset_filters(self, email, dataset_id):
         return self.provider.dataset_filters(email, dataset_id)
@@ -79,3 +80,12 @@ class DatabaseAPI:
 
     def delete_dataset_filter(self, email, dataset_id, filter_id):
         return self.provider.delete_dataset_filter(email, dataset_id, filter_id)
+
+    def upsert_feature_set(self, email, dataset_id, set_id, category, name, features):
+        return self.provider.upsert_feature_set(email, dataset_id, set_id, category, name, features)
+
+    def get_feature_sets(self, email, dataset_id):
+        return self.provider.get_feature_sets(email, dataset_id)
+
+    def delete_feature_set(self, email, dataset_id, set_id):
+        return self.provider.delete_feature_set(email, dataset_id, set_id)
