@@ -4,21 +4,36 @@ import json
 import logging
 import os
 
-import anndata
-import cirrocumulus.data_processing as data_processing
 import numpy as np
 import pandas as pd
 import pandas._libs.json as ujson
-from cirrocumulus.anndata_dataset import AnndataDataset
-from cirrocumulus.dataset_api import DatasetAPI
-from cirrocumulus.io_util import get_markers, filter_markers, add_spatial, SPATIAL_HELP
-from cirrocumulus.simple_data import SimpleData
 from natsort import natsorted
 from pandas import CategoricalDtype
 
+import cirrocumulus.data_processing as data_processing
+from cirrocumulus.anndata_dataset import AnndataDataset
+from cirrocumulus.dataset_api import DatasetAPI
+from cirrocumulus.io_util import get_markers, filter_markers, add_spatial, SPATIAL_HELP, unique_id
+from cirrocumulus.simple_data import SimpleData
+
 logger = logging.getLogger("cirro")
 
 
+def read_adata(path, backed=False, spatial_directory=None):
+    import anndata
+    adata = anndata.read_loom(path) if path.lower().endswith('.loom') else anndata.read(path,
+        backed=backed)
+    if spatial_directory is not None:
+        if not add_spatial(adata, spatial_directory):
+            print('No spatial data found in {}'.format(spatial_directory))
+    if not backed:
+        sums = adata.X.sum(axis=0)
+        if isinstance(sums, np.matrix):
+            sums = sums.A1
+        adata = adata[:, sums > 0]
+    return adata
+
+
 def to_json(data):
     return ujson.dumps(data, double_precision=2, orient='values')
 
@@ -153,45 +168,34 @@ def write_basis_obs(basis, coords_group, obs_group, result):
 
 class PrepareData:
 
-    def __init__(self, input_path, backed, output, X_range=None, dimensions=None, groups=[], markers=[],
-                 spatial_directory=None):
-        self.input_path = input_path
+    def __init__(self, adata, output, X_range=None, dimensions=None, groups=[], markers=[], output_format='parquet'):
+        self.adata = adata
         self.groups = groups
         self.markers = markers
         self.stats = False
         self.nbins = None
         self.bin_agg_function = None
-        self.output_format = 'parquet'
+        self.output_format = output_format
         # if basis_list is None or len(basis_list) == 0:
         #     basis_list = list(self.adata.obsm_keys())
         self.basis_list_to_precompute = None
-        self.adata = anndata.read_loom(input_path) if input_path.lower().endswith('.loom') else anndata.read(input_path,
-            backed=backed)
+
         # if 'seurat_clusters' in self.adata.obs:
         #     self.adata.obs['seurat_clusters'] = self.adata.obs['seurat_clusters'].astype('category')
-        if spatial_directory is not None:
-            if not add_spatial(self.adata, spatial_directory):
-                print('No spatial data found in {}'.format(spatial_directory))
-        if not backed:
-            sums = self.adata.X.sum(axis=0)
-            if isinstance(sums, np.matrix):
-                sums = sums.A1
-            self.adata = self.adata[:, sums > 0]
+
         index = make_unique(self.adata.var.index.append(pd.Index(self.adata.obs.columns)))
         self.adata.var.index = index[0:len(self.adata.var.index)]
         self.adata.obs.columns = index[len(self.adata.var.index):]
 
-        logger.info('{} - {} x {}'.format(input_path, self.adata.shape[0], self.adata.shape[1]))
         self.X_range = (0, self.adata.shape[1]) if X_range is None else X_range
         self.base_output_dir = output
 
         self.dataset_api = DatasetAPI()
         ds = AnndataDataset()
-        ds.path_to_data[input_path] = self.adata
+        ds.path_to_data[''] = self.adata
         self.dataset_api.add(ds)
 
-        self.input_dataset = {'name': os.path.splitext(os.path.basename(input_path))[0], 'url': input_path,
-                              'id': input_path}
+        self.input_dataset = {'name': '', 'url': '', 'id': ''}
         dimensions_supplied = dimensions is not None and len(dimensions) > 0
         self.dimensions = [] if not dimensions_supplied else dimensions
         self.measures = []
@@ -229,14 +233,15 @@ class PrepareData:
         X_range = self.X_range
         if not os.path.exists(self.base_output_dir):
             os.makedirs(self.base_output_dir, exist_ok=True)
-        marker_dict = self.adata.uns.get('markers', {})
-        self.adata.uns['markers'] = marker_dict
+        markers = self.adata.uns.get('markers', [])
+        self.adata.uns['markers'] = markers
         if self.groups is not None:
             n_genes = 10
             for field in self.groups:
                 if len(self.adata.obs[field].cat.categories) > 1:
                     logger.info('Computing markers for {}'.format(field))
-                    SimpleData.find_markers(self.adata, field, marker_dict, n_genes)
+                    markers += SimpleData.find_markers(self.adata, field, n_genes)
+            self.adata.uns['markers'] = markers
         images = self.adata.uns.get('images')
         if images is not None:
             image_dir = os.path.join(self.base_output_dir, 'images')
@@ -256,6 +261,8 @@ class PrepareData:
             elif self.output_format == 'json':
                 from cirrocumulus.json_io import save_adata_json
                 save_adata_json(self.adata, schema, self.base_output_dir)
+            else:
+                raise ValueError("Unknown format")
 
         # if self.stats:
         #     self.summary_stats()
@@ -346,10 +353,15 @@ class PrepareData:
         nbins = self.nbins
         bin_agg_function = self.bin_agg_function
         result = SimpleData.schema(self.adata)
-        marker_dict = self.adata.uns.get('markers', {})
+        markers = self.adata.uns.get('markers', [])
         if self.markers is not None:
-            marker_dict.update(get_markers(self.markers))
-        result['markers'] = filter_markers(self.adata, marker_dict)
+            markers += get_markers(self.markers)
+        markers = filter_markers(self.adata, markers)
+        for marker in markers:
+            if marker.get('id') is None:
+                marker['id'] = unique_id()
+            marker['readonly'] = True
+        result['markers'] = markers
         result['format'] = self.output_format
         if self.stats:
             result['precomputed'] = True  # has stats
@@ -376,7 +388,7 @@ def main(argsv):
         action='append')
     parser.add_argument('--groups', help='List of groups to compute markers for (e.g. louvain)', action='append')
     parser.add_argument('--spatial', help=SPATIAL_HELP)
-
+    # parser.add_argument('--output_format', help='Output file format', choices=['json', 'parquet'], default='parquet')
     # parser.add_argument('--nbins', help='Number of bins. Set to 0 to disable binning', default=500, type=int)
     # parser.add_argument('--summary', help='Bin summary statistic for numeric values', default='max')
     # parser.add_argument('--X_range', help='Start and end position of data matrix (e.g. 0-5000)', type=str)
@@ -389,6 +401,7 @@ def main(argsv):
     input_X_range = None
     out = args.out
     input_dataset = args.dataset
+    output_format = 'parquet'  # args.output_format
     if out is None:
         out = os.path.basename(input_dataset)
         out = out[0:out.rindex('.')]
@@ -416,8 +429,9 @@ def main(argsv):
     #     input_X_range[0] = int(input_X_range[0])
     #     input_X_range[1] = int(input_X_range[1])
 
-    prepare_data = PrepareData(input_path=input_dataset, backed=args.backed, output=out,
-        dimensions=args.groups, groups=args.groups, markers=args.markers, spatial_directory=args.spatial)
+    adata = read_adata(input_dataset, backed=args.backed, spatial_directory=args.spatial)
+    prepare_data = PrepareData(adata=adata, output=out, dimensions=args.groups, groups=args.groups,
+        markers=args.markers, output_format=output_format)
     prepare_data.execute()
     if loom_file is not None:
         os.remove(loom_file)
