@@ -2,11 +2,11 @@ import {Divider, IconButton, Menu, Tooltip} from '@material-ui/core';
 import AppBar from '@material-ui/core/AppBar';
 import Button from '@material-ui/core/Button';
 import MenuItem from '@material-ui/core/MenuItem';
-import Select from '@material-ui/core/Select';
 import {withStyles} from '@material-ui/core/styles';
 import Tab from '@material-ui/core/Tab';
 import Tabs from '@material-ui/core/Tabs';
 import Toolbar from '@material-ui/core/Toolbar';
+import Typography from '@material-ui/core/Typography';
 import AccountCircle from '@material-ui/icons/AccountCircle';
 import Brightness2Icon from '@material-ui/icons/Brightness3';
 import HelpIcon from '@material-ui/icons/Help';
@@ -28,6 +28,7 @@ import {
     setTab,
 } from './actions';
 import {drawerWidth} from './App';
+import DatasetSelector from './DatasetSelector';
 import {intFormat} from './formatters';
 import {
     DEFAULT_DARK_MODE,
@@ -136,7 +137,7 @@ class AppHeader extends React.PureComponent {
     };
 
     getLinkJson = () => {
-        const {chartOptions, primaryTraceKey, dataset, embeddings, searchTokens, datasetFilter, interpolator, markerOpacity, unselectedMarkerOpacity, dotPlotData} = this.props;
+        const {chartOptions, combineDatasetFilters, primaryTraceKey, dataset, embeddings, searchTokens, datasetFilter, interpolator, markerOpacity, unselectedMarkerOpacity, dotPlotData} = this.props;
 
         let json = {
             dataset: dataset.id,
@@ -196,8 +197,9 @@ class AppHeader extends React.PureComponent {
         }
         if (Object.keys(datasetFilterJson).length > 0) {
             json.datasetFilter = datasetFilterJson;
-        }
 
+        }
+        json.combineDatasetFilters = combineDatasetFilters;
         if (markerOpacity !== DEFAULT_MARKER_OPACITY) {
             json.markerOpacity = markerOpacity;
         }
@@ -218,10 +220,21 @@ class AppHeader extends React.PureComponent {
         return json;
     };
 
+    handleDataset = (id) => {
+        if (this.props.dataset != null) {
+            const savedDatasetState = this.props.savedDatasetState;
+            const link = this.getLinkJson();
+            link.dataset = null;
+            savedDatasetState[this.props.dataset.id] = link;
+            this.props.handleSavedDatasetState(savedDatasetState);
+        }
+        this.props.handleDataset(id);
+    };
+
     copyLink = (event) => {
 
-        let linkText = window.location.protocol + '//' + window.location.host;
-        linkText += '?q=' + JSON.stringify(this.getLinkJson());
+        let linkText = window.location.protocol + '//' + window.location.host + window.location.pathname;
+        linkText += '?q=' + encodeURIComponent(JSON.stringify(this.getLinkJson()));
         const container = document.activeElement;
         const fakeElem = document.createElement('textarea');
         const isRTL = document.documentElement.getAttribute('dir') == 'rtl';
@@ -271,16 +284,7 @@ class AppHeader extends React.PureComponent {
         this.props.handleDialog(IMPORT_DATASET_DIALOG);
         this.setState({moreMenuOpen: false});
     };
-    handleDataset = (event) => {
-        if (this.props.dataset != null) {
-            const savedDatasetState = this.props.savedDatasetState;
-            const link = this.getLinkJson();
-            link.dataset = null;
-            savedDatasetState[this.props.dataset.id] = link;
-            this.props.handleSavedDatasetState(savedDatasetState);
-        }
-        this.props.handleDataset(event.target.value);
-    };
+
     handleSettings = (event) => {
         this.props.handleDialog(EDIT_DATASET_DIALOG);
         this.setState({moreMenuOpen: false});
@@ -293,46 +297,48 @@ class AppHeader extends React.PureComponent {
 
     render() {
         const {
-            dataset, loadingApp, email, datasetChoices, selection, classes, serverInfo, tab, user
+            dataset, loadingApp, email, selection, classes, serverInfo, tab, user
         } = this.props;
-        const shape = dataset != null && dataset.shape != null ? dataset.shape : [0, 0];
-        const hasSelection = dataset != null && shape[0] > 0 && !isNaN(selection.count);
-        const showNumberOfCells = !hasSelection && dataset != null && !(selection.count > 0) && shape[0] > 0 && (selection.count !== shape[0]);
-
+        const shape = dataset != null && dataset.shape != null ? dataset.shape : null;
+        const hasSelection = dataset != null && shape != null && shape[0] > 0 && !isNaN(selection.count);
 
         const showNewDataset = user != null && user.importer;
         const showEditDeleteDataset = dataset !== null && dataset.owner;
         const showMoreMenu = showNewDataset || dataset != null;
-
+        const isSignedOut = !loadingApp.loading && email == null && serverInfo.clientId !== '';
         return (
 
             <AppBar position="fixed" color="default" className={classes.appBar}>
                 <Toolbar variant="dense">
 
+                    {/*{datasetChoices.length > 0 && <Select*/}
+                    {/*    style={{marginRight: 6}}*/}
+                    {/*    disableUnderline={true}*/}
+                    {/*    displayEmpty={true}*/}
+                    {/*    value={dataset == null ? '' : dataset.id}*/}
+                    {/*    onChange={this.handleDataset}*/}
+                    {/*    inputProps={{*/}
+                    {/*        name: 'dataset',*/}
+                    {/*        id: 'dataset-id',*/}
+                    {/*    }}*/}
+                    {/*>*/}
+                    {/*    <MenuItem key="" value="" disabled>*/}
+                    {/*        Choose a dataset*/}
+                    {/*    </MenuItem>*/}
+                    {/*    {datasetChoices.map(dataset => <MenuItem*/}
+                    {/*        key={dataset.id} value={dataset.id}>{dataset.name}</MenuItem>)}*/}
+                    {/*</Select>}*/}
 
-                    {datasetChoices.length > 0 && <Select
-                        style={{marginRight: 6}}
-                        disableUnderline={true}
-                        displayEmpty={true}
-                        value={dataset == null ? '' : dataset.id}
-                        onChange={this.handleDataset}
-                        inputProps={{
-                            name: 'dataset',
-                            id: 'dataset-id',
-                        }}
-                    >
-                        <MenuItem key="" value="" disabled>
-                            Choose a dataset
-                        </MenuItem>
-                        {datasetChoices.map(dataset => <MenuItem
-                            key={dataset.id} value={dataset.id}>{dataset.name}</MenuItem>)}
-                    </Select>}
+                    {dataset != null &&
+                    <Typography component={"h3"}>
+                        <Tooltip title={dataset.description||''}><b>{dataset.name}</b></Tooltip>
+                        <small>&nbsp;
+                            {hasSelection && shape != null && intFormat(selection.count) + ' / '}
+                            {shape != null && intFormat(shape[0]) + ' cells'}
+                        </small>
 
-                    <div className={"cirro-condensed"} style={{display: 'inline-block'}}>
-                        {hasSelection && intFormat(selection.count)}
-                        {hasSelection && ' / ' + intFormat(shape[0]) + ' cells'}
-                        {showNumberOfCells && intFormat(shape[0]) + ' cells'}
-                    </div>
+                    </Typography>
+                    }
 
                     {dataset != null && <Tabs
                         value={tab}
@@ -354,7 +360,7 @@ class AppHeader extends React.PureComponent {
 
                     </div>
                     <div style={{marginLeft: 'auto'}}>
-
+                        {!isSignedOut && <DatasetSelector onChange={this.handleDataset}/>}
                         {showMoreMenu && <Tooltip title={'More'}>
                             <IconButton aria-label="Menu" aria-haspopup="true"
                                         onClick={this.handleMoreMenuOpen}>
@@ -420,9 +426,8 @@ class AppHeader extends React.PureComponent {
                         </Menu>}
 
 
-                        {!loadingApp.loading && email == null && serverInfo.clientId !== '' &&
-                        <Button style={{whiteSpace: 'nowrap'}} variant="outlined" color="primary"
-                                onClick={this.props.handleLogin}>Sign In</Button>}
+                        {isSignedOut && <Button style={{whiteSpace: 'nowrap'}} variant="outlined" color="primary"
+                                                onClick={this.props.handleLogin}>Sign In</Button>}
                     </div>
                 </Toolbar>
             </AppBar>
