@@ -6,11 +6,12 @@ import {DotPlotGroup} from './DotPlotGroup';
 
 class DotPlots extends React.PureComponent {
     render() {
-        const {dotPlotData, categoricalNames, selectedDotPlotData} = this.props;
+        const {chartOptions, dotPlotData, categoricalNames, onSortOrderChanged, selectedDotPlotData} = this.props;
 
         if (dotPlotData.length === 0) {
             return <h4>Please enter one or more categorical observations and one or more features.</h4>;
         }
+        const textColor = chartOptions.darkMode ? 'white' : 'black';
         let selectedDotPlotNameToData = {};
         selectedDotPlotData.forEach(categoryItem => {
             selectedDotPlotNameToData[categoryItem.name] = categoryItem;
@@ -19,8 +20,12 @@ class DotPlots extends React.PureComponent {
             {dotPlotData.map((categoryItem) => {
                 let selectedData = selectedDotPlotNameToData[categoryItem.name];
                 let renamedCategories = categoricalNames[categoryItem.name];
-                return <DotPlotGroup key={categoryItem.name} categoryItem={categoryItem} selectedData={selectedData}
-                                     renamedCategories={renamedCategories}/>;
+                return <DotPlotGroup key={categoryItem.name}
+                                     categoryItem={categoryItem}
+                                     selectedData={selectedData}
+                                     onSortOrderChanged={onSortOrderChanged}
+                                     renamedCategories={renamedCategories}
+                                     textColor={textColor}/>;
             })}
         </div>;
     }
@@ -28,6 +33,8 @@ class DotPlots extends React.PureComponent {
 
 const mapStateToProps = state => {
     return {
+        chartOptions: state.chartOptions,
+
         dotPlotData: state.dotPlotData,
         selectedDotPlotData: state.selectedDotPlotData,
         categoricalNames: state.categoricalNames
