@@ -31,17 +31,14 @@ export class RestServerApi {
             });
     }
 
-    upsertDatasetPromise(datasetId, url, name, readers) {
+    upsertDatasetPromise(datasetId, dataset) {
         let isEdit = datasetId != null;
+        if (datasetId != null) {
+            dataset.id = datasetId;
+        }
         return fetch(API + '/dataset',
             {
-                body: JSON.stringify(
-                    {
-                        id: datasetId,
-                        url: url,
-                        name: name,
-                        readers: readers
-                    }),
+                body: JSON.stringify(dataset),
                 method: isEdit ? 'PUT' : 'POST',
                 headers: {'Authorization': 'Bearer ' + getIdToken()},
             }).then(importDatasetResponse => importDatasetResponse.json());
@@ -76,6 +73,25 @@ export class RestServerApi {
             }).then(response => response.json());
     }
 
+    upsertFeatureSet(data, isUpdate) {
+        return fetch(API + '/feature_set',
+            {
+                body: JSON.stringify(data),
+                method: isUpdate ? 'PUT' : 'POST',
+                headers: {'Authorization': 'Bearer ' + getIdToken()},
+            }).then(response => response.json());
+    }
+
+    deleteFeatureSet(setId, datasetId) {
+        return fetch(API + '/feature_set',
+            {
+                body: JSON.stringify({id: setId, ds_id: datasetId}),
+                method: 'DELETE',
+                headers: {'Authorization': 'Bearer ' + getIdToken()},
+            });
+    }
+
+
     exportDatasetFiltersPromise(datasetId) {
         return fetch(API + '/export_filters?id=' + datasetId, {
             headers: {'Authorization': 'Bearer ' + getIdToken()},
