@@ -1,11 +1,11 @@
 import {scaleOrdinal, scaleSequential} from 'd3-scale';
 import {schemeCategory10, schemePaired} from 'd3-scale-chromatic';
 import {saveAs} from 'file-saver';
-import {isEqual, isPlainObject, uniqBy} from 'lodash';
+import {isEqual, uniqBy} from 'lodash';
 import natsort from 'natsort';
 import OpenSeadragon from 'openseadragon';
-import {ByteRangeDataset} from '../ByteRangeDataset';
 import CustomError from '../CustomError';
+import {JsonDataset} from '../JsonDataset';
 import {RestDataset} from '../RestDataset';
 import {RestServerApi} from '../RestServerApi';
 
@@ -13,9 +13,10 @@ import {RestServerApi} from '../RestServerApi';
 import {getPositions} from '../ThreeUtil';
 
 import {
+    addFeatureSetsToX,
     CATEGORY_20B,
     CATEGORY_20C,
-    convertBinsToPoints,
+    convertBinsToPoints, getFeatureSets,
     getInterpolator,
     indexSort,
     randomSeq,
@@ -24,7 +25,7 @@ import {
 } from '../util';
 
 // export const API = 'http://localhost:5000/api';
-export const API = '/api';
+export const API = 'api';
 
 const authScopes = [
     'email',
@@ -78,12 +79,14 @@ export const SET_POINT_SIZE = 'SET_POINT_SIZE';
 export const SET_EMAIL = 'SET_EMAIL';
 export const SET_USER = 'SET_USER';
 export const SET_DATASET = 'SET_DATASET';
+export const SET_MARKERS = 'SET_MARKERS';
 export const SET_DIALOG = 'SET_DIALOG';
 
 export const MORE_OPTIONS_DIALOG = 'MORE_OPTIONS_DIALOG';
 export const EDIT_DATASET_DIALOG = 'EDIT_DATASET_DIALOG';
 export const IMPORT_DATASET_DIALOG = 'IMPORT_DATASET_DIALOG';
 export const SAVE_DATASET_FILTER_DIALOG = 'SAVE_DATASET_FILTER_DIALOG';
+export const SAVE_FEATURE_SET_DIALOG = 'SAVE_FEATURE_SET_DIALOG';
 export const HELP_DIALOG = 'HELP_DIALOG';
 export const DELETE_DATASET_DIALOG = 'DELETE_DATASET_DIALOG';
 
@@ -279,6 +282,65 @@ export function deleteDatasetFilter(filterId) {
     };
 }
 
+export function deleteFeatureSet(id) {
+    return function (dispatch, getState) {
+        dispatch(_setLoading(true));
+        getState().serverInfo.api.deleteFeatureSet(id, getState().dataset.id)
+            .then(result => {
+
+                let markers = getState().markers;
+                for (let i = 0; i < markers.length; i++) {
+                    if (markers[i].id === id) {
+                        markers.splice(i, 1);
+                        // remove from searchTokens
+                        break;
+                    }
+                }
+                dispatch(setMarkers(markers.slice()));
+                dispatch(setMessage('Set deleted'));
+            }).finally(() => {
+            dispatch(_setLoading(false));
+        }).catch(err => {
+            handleError(dispatch, err, 'Unable to delete set. Please try again.');
+        });
+    };
+}
+
+export function saveFeatureSet(payload) {
+    return function (dispatch, getState) {
+        const state = getState();
+        const searchTokens = state.searchTokens;
+        const splitTokens = splitSearchTokens(searchTokens);
+        let features = splitTokens.X;
+        const requestBody = {
+            ds_id: state.dataset.id,
+            name: payload.name,
+            features: features,
+            category: payload.category
+        };
+
+        dispatch(_setLoading(true));
+        getState().serverInfo.api.upsertFeatureSet(requestBody, false)
+            .then(result => {
+
+                let markers = getState().markers;
+                markers.push({
+                    category: payload.category,
+                    name: payload.name,
+                    features: features,
+                    id: result.id
+                });
+                dispatch(setMarkers(markers.slice()));
+                dispatch(setMessage('Set saved'));
+            }).finally(() => {
+            dispatch(_setLoading(false));
+            dispatch(setDialog(null));
+        }).catch(err => {
+            handleError(dispatch, err, 'Unable to save set. Please try again.');
+        });
+    };
+}
+
 export function saveDatasetFilter(payload) {
     return function (dispatch, getState) {
         const state = getState();
@@ -501,7 +563,7 @@ function handleFilterUpdated() {
 
         const searchTokens = splitSearchTokens(state.searchTokens);
         let filter = getFilterJson(state);
-
+        addFeatureSetsToX(getFeatureSets(state.markers, searchTokens.featureSets), searchTokens.X);
         let q = {
             selection: {
                 measures: searchTokens.X.concat(searchTokens.obs.map(item => 'obs/' + item)),
@@ -624,13 +686,13 @@ function _handleCategoricalNameChange(payload) {
 export function handleCategoricalNameChange(payload) {
     return function (dispatch, getState) {
         const dataset_id = getState().dataset.id;
-        const p = getState().serverInfo.fancy? getState().serverInfo.api.setCategoryNamePromise({
-                c: payload.name,
-                o: payload.oldValue,
-                n: payload.value,
-                id: dataset_id
-            }):Promise.resolve();
-       p.then(() => dispatch(_handleCategoricalNameChange(payload)));
+        const p = getState().serverInfo.fancy ? getState().serverInfo.api.setCategoryNamePromise({
+            c: payload.name,
+            o: payload.oldValue,
+            n: payload.value,
+            id: dataset_id
+        }) : Promise.resolve();
+        p.then(() => dispatch(_handleCategoricalNameChange(payload)));
     };
 }
 
@@ -874,13 +936,15 @@ export function getIdToken() {
 
 export function saveDataset(payload) {
     return function (dispatch, getState) {
-        const readers = payload.readers;
+
         const name = payload.name;
         const url = payload.url;
 
-        if (name == '' || url === '') {
+        if (name === '' || url === '') {
             return;
         }
+        const readers = payload.readers;
+        const description = payload.description;
         // let bucket = url.substring('gs://'.length);
         // let slash = bucket.indexOf('/');
         // let object = encodeURIComponent(bucket.substring(slash + 1));
@@ -913,12 +977,18 @@ export function saveDataset(payload) {
             // if (!permissionsResponse.ok) {
             //     dispatch(setMessage('Unable to set dataset read permissions. Please ensure that you are the dataset owner or manually add ' + serverEmail + ' as a reader.'));
             // }
-        }).then(() => getState().serverInfo.api.upsertDatasetPromise(payload.dataset ? payload.dataset.id : null, url, name, readers)).then(importDatasetResult => {
+        }).then(() => getState().serverInfo.api.upsertDatasetPromise(payload.dataset ? payload.dataset.id : null,
+            {name: name, readers: readers, description: description, url: url})).then(importDatasetResult => {
             if (isEdit) {
-                dispatch(updateDataset({name: name, id: importDatasetResult.id, owner: true}));
+                dispatch(updateDataset({
+                    name: name,
+                    id: importDatasetResult.id,
+                    description: description,
+                    owner: true
+                }));
                 dispatch(setMessage('Dataset updated'));
             } else {
-                dispatch(_addDataset({name: name, id: importDatasetResult.id, owner: true}));
+                dispatch(_addDataset({name: name, id: importDatasetResult.id, description: description, owner: true}));
                 if (serverInfo.email) {
                     dispatch(setMessage(updatePermissions ? 'Please ensure that ' + serverInfo.email + ' is a "Storage Object Viewer"' : 'Dataset created'));
                 }
@@ -978,6 +1048,10 @@ export function setDialog(payload) {
     return {type: SET_DIALOG, payload: payload};
 }
 
+export function setMarkers(payload) {
+    return {type: SET_MARKERS, payload: payload};
+}
+
 function _setDatasetChoices(payload) {
     return {type: SET_DATASET_CHOICES, payload: payload};
 }
@@ -1115,22 +1189,19 @@ export function setDataset(id, loadDefaultView = true, setLoading = true) {
             return Promise.reject('Unable to find dataset');
         }
         // force re-render selected dataset dropdown
-        let dataset = {
-            owner: selectedChoice.owner,
-            name: selectedChoice.name,
-            id: id,
-            url: selectedChoice.url,
-            embeddings: [],
-            features: [],
-            obs: [],
-            obsCat: []
-        };
+        let dataset = Object.assign({}, selectedChoice);
+        dataset.id = id;
+        dataset.embeddings = [];
+        dataset.features = [];
+        dataset.obs = [];
+        dataset.obsCat = [];
         dispatch(_setDataset(dataset));
         let categoryNameResults;
         let datasetFilters = [];
         let newDataset;
 
         function onPromisesComplete() {
+            newDataset = Object.assign(dataset, newDataset);
             if (newDataset.embeddings) {
                 for (let i = 0; i < newDataset.embeddings.length; i++) {
                     if (newDataset.embeddings[i].nbins != null) {
@@ -1145,8 +1216,6 @@ export function setDataset(id, loadDefaultView = true, setLoading = true) {
                 }
             }
             newDataset.api = dataset.api;
-            newDataset.owner = selectedChoice.owner;
-            newDataset.name = selectedChoice.name;
             newDataset.features = newDataset.var;
             newDataset.features.sort((a, b) => {
                 a = a.toLowerCase();
@@ -1186,7 +1255,7 @@ export function setDataset(id, loadDefaultView = true, setLoading = true) {
         let url = dataset.url || '';
         const isByteRangeDataset = url.startsWith("http") || url.startsWith('//');
         if (isByteRangeDataset) {
-            dataset.api = new ByteRangeDataset();
+            dataset.api = new JsonDataset();
         } else {
             dataset.api = new RestDataset();
         }
@@ -1270,11 +1339,10 @@ function handleSelectionResult(selectionResult, clear) {
             }
 
             // userPoints are in chart space, points are in server space, count is total number of cells selected
-
             if (selectionResult.summary) {
                 // merge or clear selection
                 let selectionSummary = clear ? selectionResult.summary : Object.assign(getState().featureSummary, selectionResult.summary);
-                dispatch(setFeatureSummary(selectionSummary));
+                dispatch(setFeatureSummary(Object.assign({}, selectionSummary)));
             }
             if (selectionResult.dotplot) {
                 let selectedDotPlotData = state.selectedDotPlotData;
@@ -1300,6 +1368,7 @@ function handleSelectionResult(selectionResult, clear) {
                 });
                 dispatch(_setSelectedDotPlotData(selectedDotPlotData.slice()));
             }
+
         }
     };
 
@@ -1315,20 +1384,7 @@ function _updateCharts(onError) {
 
         const searchTokens = splitSearchTokens(state.searchTokens);
         if (searchTokens.featureSets.length > 0) {
-            // add to X, maintaining insertion order
-            const uniqueX = new Set(searchTokens.X );
-            searchTokens.featureSets.forEach(featureSet => {
-                const groupMarkers = state.dataset.markers[featureSet.group];
-                const features = groupMarkers[featureSet.text];
-                features.forEach(feature=>{
-                    if(!uniqueX.has(feature)){
-                        searchTokens.X.push(feature);
-                        uniqueX.add(feature);
-                    }
-                });
-
-            });
-
+            addFeatureSetsToX(getFeatureSets(state.markers, searchTokens.featureSets), searchTokens.X);
         }
         let dotplot = (searchTokens.X.length > 0 || searchTokens.featureSets.length > 0) && searchTokens.obsCat.length > 0;
         if (searchTokens.X.length === 0 && searchTokens.obsCat.length === 0 && searchTokens.obs.length === 0
@@ -1363,7 +1419,8 @@ function _updateCharts(onError) {
         });
 
         let dotplotMeasures = new Set();
-        let dotplotDimensions = [];
+        let dotPlotCategories = searchTokens.obsCat.slice();
+        let dotPlotCategoryKeys = [dotPlotCategories.join('-')];
         let dotPlotKeys = {}; // category-feature
         if (dotplot) {
             let cachedDotPlotKeys = {};
@@ -1372,7 +1429,7 @@ function _updateCharts(onError) {
                     cachedDotPlotKeys[dotPlotDataItem.name + '-' + datum.name] = true;
                 });
             });
-            searchTokens.obsCat.forEach(category => {
+            dotPlotCategoryKeys.forEach(category => {
                 let added = false;
                 searchTokens.X.forEach(feature => {
                     if (feature !== '__count') {
@@ -1384,9 +1441,9 @@ function _updateCharts(onError) {
                         }
                     }
                 });
-                if (added) {
-                    dotplotDimensions.push(category);
-                }
+                // if (added) {
+                //     dotplotDimensions.push(category);
+                // }
             });
         }
 
@@ -1468,7 +1525,9 @@ function _updateCharts(onError) {
                 });
             }
 
-            dotPlotData = dotPlotData.filter(categoryItem => searchTokens.obsCat.indexOf(categoryItem.name) !== -1);
+            // keep active categories only
+            let categoryKeys = [searchTokens.obsCat.join('-')];
+            dotPlotData = dotPlotData.filter(categoryItem => categoryKeys.indexOf(categoryItem.name) !== -1);
             dotPlotData.forEach((categoryItem, categoryIndex) => {
                 const nfeatures = categoryItem.values.length;
                 categoryItem.values = categoryItem.values.filter(feature => dotPlotKeys[categoryItem.name + '-' + feature.name]);
@@ -1500,14 +1559,14 @@ function _updateCharts(onError) {
 
             // sort features
             let featureSortOrder = {};
-            searchTokens.X.forEach((name, index)=>{
+            searchTokens.X.forEach((name, index) => {
                 featureSortOrder[name] = index;
             });
             dotPlotData.forEach((categoryItem) => {
-                categoryItem.values.sort((a, b)=>{
+                categoryItem.values.sort((a, b) => {
                     a = featureSortOrder[a.name];
                     b = featureSortOrder[b.name];
-                    return a-b;
+                    return a - b;
                 });
 
             });
@@ -1515,10 +1574,10 @@ function _updateCharts(onError) {
         }
 
 
-        if (dotplotDimensions.length > 0 && dotplotMeasures.size > 0) {
+        if (dotPlotCategories.length > 0 && dotplotMeasures.size > 0) {
             q.groupedStats = {
                 measures: Array.from(dotplotMeasures),
-                dimensions: dotplotDimensions
+                dimensions: [dotPlotCategories]
             };
         }
 
@@ -1637,7 +1696,7 @@ function handleEmbeddingResult(embedding, embeddingDef) {
 
             let values = embeddingValues[name];
             let purity = null;
-            if (isPlainObject(values)) {
+            if (values.value !== undefined) {
                 purity = values.purity;
                 values = values.value;
             }
@@ -1726,7 +1785,6 @@ function handleEmbeddingResult(embedding, embeddingDef) {
                     crossOriginPolicy: "Anonymous"
                 });
                 chartData.tileSource = tileSource;
-                // chartData.image = fetch(API + '/file?id=' + state.dataset.id + '&file=' + selectedEmbedding.image, {headers: {'Authorization': 'Bearer ' + getIdToken()}});
             }
             embeddingData.push(chartData);
 
@@ -1753,11 +1811,6 @@ export function listDatasets() {
         dispatch(_setLoading(true));
         return getState().serverInfo.api.getDatasetsPromise()
             .then(choices => {
-                choices.sort((a, b) => {
-                    a = a.name.toLowerCase();
-                    b = b.name.toLowerCase();
-                    return a < b ? -1 : (a === b ? 0 : -1);
-                });
                 dispatch(_setDatasetChoices(choices));
             })
             .finally(() => {
