@@ -1,33 +1,18 @@
 import {isObject} from 'lodash';
-import {getBasis} from './ByteRangeDataset';
+import {getBasis} from './JsonDataset';
 import {getIndices} from './VectorUtil';
 
-function combineAnd(array1, array2) {
-    let result = new Set();
-    for (let i = 0, n = array1.length; i < n; i++) {
-        if (array1[i] && array2[i]) {
-            result.add(i);
-        }
-    }
-    return Array.from(result);
+function combine(a, b, op) {
+    return op === 'or' ? new Set([...a, ...b]) : new Set(
+        [...a].filter(x => b.has(x)));
 }
 
-function combineOr(array1, array2) {
-    let result = new Set();
-    for (let i = 0, n = array1.length; i < n; i++) {
-        if (array1[i] || array2[i]) {
-            result.add(i);
-        }
-    }
-    return Array.from(result);
-}
 
 export function getPassingFilterIndices(df, data_filter) {
     let passingIndices = null;
     if (data_filter) {
         let user_filters = data_filter.filters || [];
         let combine_filters = data_filter.combine || 'and';
-
         for (let i = 0; i < user_filters.length; i++) {
             let filter_obj = user_filters[i];
             let field = filter_obj[0];
@@ -55,31 +40,31 @@ export function getPassingFilterIndices(df, data_filter) {
                         let p = path[j];
                         let xKeep = getIndices(df[coordinate_columns[0]], (val) => val >= p.x && val <= p.x + p.width);
                         let yKeep = getIndices(df[coordinate_columns[1]], (val) => val >= p.y && val <= p.y + p.height);
-                        selection_keep = combineAnd(xKeep, yKeep);
+                        selection_keep = combine(xKeep, yKeep, 'and');
                         if (p.z) {  // 3d
                             let zKeep = getIndices(df[coordinate_columns[2]], (val) => val >= p.z && val <= p.z + p.depth);
-                            selection_keep = combineAnd(selection_keep, zKeep);
+                            selection_keep = combine(selection_keep, zKeep, 'and');
                         }
                     }
-                    keep = keep ? combineOr(selection_keep, keep) : selection_keep;
+                    keep = keep ? combine(selection_keep, keep, combine_filters) : selection_keep;
                 }
             } else {
                 let series = df[field];
                 let applyFunction;
-                if (op == 'in') {
+                if (op === 'in') {
                     value = new Set(value);
                     applyFunction = (d) => value.has(d);
-                } else if (op == '>') {
+                } else if (op === '>') {
                     applyFunction = (d) => d > value;
-                } else if (op == '=') {
+                } else if (op === '=') {
                     applyFunction = (d) => d === value;
-                } else if (op == '<') {
+                } else if (op === '<') {
                     applyFunction = (d) => d < value;
-                } else if (op == '!=') {
+                } else if (op === '!=') {
                     applyFunction = (d) => d !== value;
-                } else if (op == '>=') {
+                } else if (op === '>=') {
                     applyFunction = (d) => d >= value;
-                } else if (op == '<=') {
+                } else if (op === '<=') {
                     applyFunction = (d) => d <= value;
                 } else {
                     throw('Unknown filter');
@@ -89,15 +74,12 @@ export function getPassingFilterIndices(df, data_filter) {
 
 
             if (passingIndices) {
-                if (combine_filters == 'and') {
-                    passingIndices = combineAnd(passingIndices, keep);
-                } else {
-                    passingIndices = combineOr(passingIndices, keep);
-                }
+                passingIndices = combine(passingIndices, keep, combine_filters);
             } else {
                 passingIndices = keep;
             }
         }
     }
-    return passingIndices;
+
+    return passingIndices != null ? Array.from(passingIndices) : null;
 }
