@@ -1,4 +1,5 @@
 import {scaleSequential} from 'd3-scale';
+import {isPlainObject} from 'lodash';
 import {combineReducers} from 'redux';
 import {
     ADD_DATASET,
@@ -29,6 +30,7 @@ import {
     SET_LOADING_APP,
     SET_MARKER_OPACITY,
     SET_MARKER_OPACITY_UI,
+    SET_MARKERS,
     SET_MESSAGE,
     SET_NUMBER_OF_BINS,
     SET_NUMBER_OF_BINS_UI,
@@ -140,6 +142,63 @@ function dataset(state = null, action) {
         case SET_DATASET:
             document.title = action.payload == null ? 'Cirro' : action.payload.name + ' - Cirro';
             return action.payload;
+        case UPDATE_DATASET:
+            if (action.payload.id === state.id) { // update name, description, url
+                document.title = action.payload.name + ' - Cirro';
+                return Object.assign(state, action.payload);
+            }
+        default:
+            return state;
+    }
+}
+
+function datasetChoices(state = [], action) {
+    switch (action.type) {
+        case SET_DATASET_CHOICES:
+            action.payload.sort((a, b) => {
+                a = a.name.toLowerCase();
+                b = b.name.toLowerCase();
+                return a === b ? 0 : (a < b ? -1 : 1);
+            });
+
+            return action.payload;
+        case SET_EMAIL:
+            if (action.payload == null) {
+                return [];
+            }
+            return state;
+        case ADD_DATASET:
+            state.push(action.payload);
+            state.sort((a, b) => {
+                a = a.name.toLowerCase();
+                b = b.name.toLowerCase();
+                return a === b ? 0 : (a < b ? -1 : 1);
+            });
+            return state.slice();
+        case UPDATE_DATASET:
+        case DELETE_DATASET:
+            let index = -1;
+            for (let i = 0; i < state.length; i++) {
+                if (state[i].id === action.payload.id) {
+                    index = i;
+                    break;
+                }
+            }
+            if (index !== -1) {
+                if (action.type === UPDATE_DATASET) {
+                    state[index] = action.payload;
+                    state.sort((a, b) => {
+                        a = a.name.toLowerCase();
+                        b = b.name.toLowerCase();
+                        return a === b ? 0 : (a < b ? -1 : 1);
+                    });
+                } else {
+                    state.splice(index, 1);
+                }
+
+                return state.slice();
+            }
+            return state;
         default:
             return state;
     }
@@ -159,6 +218,63 @@ function embeddings(state = [], action) {
     }
 }
 
+function markers(state = [], action) {
+    switch (action.type) {
+        case SET_MARKERS:
+            return action.payload;
+        case SET_DATASET:
+            let result = action.payload == null ? [] : action.payload.markers || [];
+            if (isPlainObject(result)) { // old style, category=>name=>features
+                let newResults = [];
+                for (let categoryName in result) {
+                    let category = result[categoryName];
+                    for (let name in category) {
+                        newResults.push({
+                            category: categoryName,
+                            name: name,
+                            id: categoryName + '-' + name,
+                            readonly: true,
+                            features: category[name]
+                        });
+                    }
+                }
+                result = newResults;
+            }
+
+            if (action.payload.markers_read_only != null) {
+                if (isPlainObject(action.payload.markers_read_only)) { // old style, name => features
+                    let markers = action.payload.markers_read_only;
+                    for (let categoryName in markers) {
+                        let category = markers[categoryName];
+                        for (let name in category) {
+                            result.push({
+                                category: categoryName,
+                                name: name,
+                                id: categoryName + '-' + name,
+                                readonly: true,
+                                features: category[name]
+                            });
+                        }
+                    }
+                } else {
+                    action.payload.markers_read_only.forEach(item => {
+                        item.readonly = true;
+                        result.push(item);
+                    });
+                }
+            }
+            result.forEach(item => {
+                if (item.id == null) {
+                    item.id = item.category + '-' + item.name;
+                }
+            });
+
+            return result;
+        default:
+            return state;
+    }
+}
+
 
 function binSummary(state = DEFAULT_BIN_SUMMARY, action) {
     switch (action.type) {
@@ -365,41 +481,6 @@ function loadingApp(state = {loading: false, progress: 0}, action) {
     }
 }
 
-function datasetChoices(state = [], action) {
-    switch (action.type) {
-        case SET_DATASET_CHOICES:
-            return action.payload;
-        case SET_EMAIL:
-            if (action.payload == null) {
-                return [];
-            }
-            return state;
-        case ADD_DATASET:
-            state = state.concat([action.payload]);
-            state.sort((a, b) => a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1);
-            return state;
-        case UPDATE_DATASET:
-        case DELETE_DATASET:
-            let index = -1;
-            for (let i = 0; i < state.length; i++) {
-                if (state[i].id === action.payload.id) {
-                    index = i;
-                    break;
-                }
-            }
-            if (index !== -1) {
-                if (action.type === UPDATE_DATASET) {
-                    state[index] = action.payload;
-                } else {
-                    state.splice(index, 1);
-                }
-                return state.slice();
-            }
-            return state;
-        default:
-            return state;
-    }
-}
 
 function tab(state = 'embedding', action) {
     switch (action.type) {
@@ -699,6 +780,7 @@ export default combineReducers({
     dialog,
     dotPlotData,
     email,
+    markers,
     embeddingData,
     embeddings,
     featureSummary,
